<?php

namespace Drupal\email_change_verification\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\email_change_verification\EmailChangeServiceInterface;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class MailChangeController.
 *
 * @package Drupal\email_change_verification\Controller
 */
class EmailChangeController extends ControllerBase {

  /**
   * The Email change service.
   *
   * @var \Drupal\email_change_verification\EmailChangeServiceInterface
   */
  protected $emailChangeService;

  /**
   * EmailChangeController constructor.
   *
   * @param \Drupal\email_change_verification\EmailChangeServiceInterface $emailChangeService
   *   The Email change service.
   */
  public function __construct(EmailChangeServiceInterface $emailChangeService) {
    $this->emailChangeService = $emailChangeService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('email_change_verification.service')
    );
  }

  /**
   * Handles the email verification.
   *
   * In order to never disclose a mail change link via a referrer header this
   * controller must always return a redirect response.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user account for which the email change was requested.
   * @param string $new_email
   *   The newly requested email address.
   * @param int $timestamp
   *   The timestamp of the email change request.
   * @param string $hash
   *   Unique hash.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   An HTTP response doing a redirect.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function verify(UserInterface $user, $new_email, $timestamp, $hash) {
    // If the user is not logged in, redirect to the login page with
    // the correct destination.
    /** @var \Drupal\Core\Session\AccountProxyInterface $current_user */
    $current_user = $this->currentUser();
    if ($current_user->isAnonymous()) {
      $this->messenger()->addError($this->t('You must be logged in to verify your email address.'));
      return $this->redirect('user.login', [], ['query' => $this->getDestinationArray()]);
    }

    // If the current user does not equal the user for which to change the
    // address, throw an error.
    if ($current_user->id() !== $user->id()) {
      $this->messenger()->addError($this->t(
        'You are currently logged in as %user, and are attempting to verify an email address change for another account. Please <a href=":logout">log out</a> and try using the link again.',
        [
          '%user' => $current_user->getAccountName(),
          ':logout' => Url::fromRoute('user.logout')->toString(),
        ]
      ));
      return $this->redirect('<front>');
    }

    // If the link has expired, throw an error.
    $timeout = $this->config('email_change_verification.config')->get('timeout');
    if ($timestamp + $timeout < time()) {
      $this->messenger()->addError($this->t('You have tried to use an email address verification link that has expired. Please visit your account and change your email again.'));
      return $this->redirect('<front>');
    }

    // If the hash is not correct, throw an error.
    if (!$this->emailChangeService->checkHash($user, $new_email, $timestamp, $hash)) {
      $this->messenger()->addError($this->t('You have tried to use an email address verification link that has either been used or is no longer valid. Please visit your account and change your email again.'));
      return $this->redirect('<front>');
    }

    // If none of the above errors occurred, change the email address and to the
    // front page.
    if ($this->emailChangeService->isUnique($new_email)) {
      $user->setEmail($new_email);
      $user->set('mail_change', '');
      $user->save();
      $this->messenger()->addMessage($this->t('Your email address has been changed to %mail.', ['%mail' => $new_email]));
    }
    else {
      $this->messenger()->addError($this->t('You have tried to use an email address that is already in use.'));
    }
    return $this->redirect('<front>');
  }

}
